﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
        Based on the CLab source code developed by Rune Møller Jensen.
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

using CLab.Exceptions;

namespace CLab.BDD
{
    /// <summary>
    /// Class representing the enumeration type.
    /// </summary>
    public class BDDTypeEnum : BDDType
    {
     
        private ArrayList indexToValue;
        private Hashtable valueToIndex;
       
        /// <summary>
        /// Initializes a new instance of the <see cref="T:BDDLayoutTypeEnum"/> class.
        /// </summary>
        /// <param name="typeName">Name of the type.</param>
        /// <param name="domainSize">Size of the domain.</param>
        /// <param name="indexToValue">List mapping index to value.</param>
        /// <param name="valueToIndex">Hashtable mapping value to index.</param>
        public BDDTypeEnum(String typeName, int domainSize, ArrayList indexToValue, Hashtable valueToIndex)
            : base(typeName, BooleanVarNumber(domainSize), domainSize)
        {
            this.indexToValue = indexToValue;
            this.valueToIndex = valueToIndex;
        }

        /// <summary>
        /// Gets the hashtable mapping value to index.
        /// </summary>
         public Hashtable ValueToIndex
        {
            get { return valueToIndex; }
        }

        /// <summary>
        /// Gets the list mapping index to value.
        /// </summary>
        /// <value>The index to value.</value>
        public ArrayList IndexToValue
        {
            get { return indexToValue; }
        }

        /// <summary>
        /// Returns the <see cref="F:CPTypes.enum_type"></see> constant.
        /// </summary>
        /// <returns>The <see cref="F:CPTypes.enum_type"></see> constant.</returns>
        public override CPTypes TypeFormat()
        {
            return CPTypes.enum_type;
        }

        /// <summary>
        /// Method that gets the Domain value at "index".
        /// Index can be "0" to "domain size - 1".
        /// </summary>
        /// <param name="index">The index of the domain value.</param>
        /// <returns>
        /// The string representation of the domain value.
        /// </returns>
        /// <exception cref="CLab.Exceptions.ClabException"/>
        public override string GetDomainValue(int index)
        {
            if (index >= 0 && index < DomainSize)
            {
                return (String) indexToValue[index];
            }
            else throw new ClabException("Index is out of the BDDTypeEnum's range");
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:CLab.BDD.BDDLayoutTypeEnum"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:CLab.BDD.BDDLayoutTypeEnum"></see>.
        /// </returns>
        public override string ToString()
        {
            String indexToValuePrint = "";

                indexToValuePrint += "'indexToValue:' ";
                for (int i = 0; i < indexToValue.Count; i++)
                {
                    indexToValuePrint += "" + indexToValue[i] + " = " + i + ", ";
                }
            
           return string.Format("<LayoutTypeEnumeration> {0} \n{1}", base.ToString(), indexToValuePrint);
        }
    }
}


  
        
     
       
