﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
        Based on the CLab source code developed by Rune Møller Jensen.
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

using CLab.Exceptions;

namespace CLab.BDD
{
    /// <summary>
    /// Class representing the range type.
    /// </summary>
    public class BDDTypeRange : BDDType
    {
        private int startOfRange;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:BDDLayoutTypeRange"/> class.
        /// </summary>
        /// <param name="typeName">Name of the type.</param>
        /// <param name="startOfRange">The start of the range.</param>
        /// <param name="domainSize">Size of the domain.</param>
        public BDDTypeRange(String typeName, int startOfRange, int domainSize)
            : base(typeName, BooleanVarNumber(domainSize), domainSize)
        {
            this.startOfRange = startOfRange;
        }

        /// <summary>
        /// Gets the start of the range.
        /// </summary>
        /// <value>The start of the range.</value>
        public int StartOfRange
        {
            get { return startOfRange; }
        }

        /// <summary>
        /// Returns the <see cref="F:CPTypes.range_type"></see> constant..
        /// </summary>
        /// <returns>The <see cref="F:CPTypes.range_type"></see> constant.</returns>
        public override CPTypes TypeFormat()
        {
            return CPTypes.range_type;
        }

        /// <summary>
        /// Method that gets the Domain value at "index".
        /// Index can be "0" to "domain size - 1".
        /// </summary>
        /// <param name="index">The index of the domain value.</param>
        /// <returns>
        /// The string representation of the domain value.
        /// </returns>
        /// <exception cref="CLab.Exceptions.ClabException"/>
        public override string GetDomainValue(int index)
        {
            if (index >= 0 && index < DomainSize)
            {
                int value = startOfRange + index;
                return "" + value;
            }
            else throw new ClabException("Index is out of the BDDTypeEnum's range");
        }


        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:CLab.BDD.BDDLayoutRange"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:CLab.BDD.BDDLayoutRange"></see>.
        /// </returns>
        public override string ToString()
        {
            return (String.Format("<LayoutTypeRange>, Start of range: {0}, {1}", StartOfRange, base.ToString()));
        }
    }
}