﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
        Based on the CLab source code developed by Rune Møller Jensen.
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace CLab.BDD
{
    /// <summary>
    /// Class representing a CLab variable.
    /// </summary>
    public class BDDVariable
    {
        List<int> bddVar = new List<int>();
        int typeIndex;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:BDDLayoutVariable"/> class.
        /// </summary>
        /// <param name="bddVar">The BDD variables, which together makes a Clab variable.</param>
        /// <param name="typeIndex">Index of belonging type</param>
        public BDDVariable(List<int> bddVar, int typeIndex)
        {
            this.bddVar = bddVar;
            this.typeIndex = typeIndex;
        }

        /// <summary>
        /// Gets the BDD variable list.
        /// </summary>
        /// <value>The BDD variable list.</value>
        public List<int> BddVar
        {
            get { return bddVar; }
        }

        /// <summary>
        /// Gets the index of the belonging type.
        /// </summary>
        /// <value>The index of the type.</value>
        public int TypeIndex
        {
            get { return typeIndex; }
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:CLab.Data.BDDLayoutVariable"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:CLab.Data.BDDLayoutVariable"></see>.
        /// </returns>
        public override string ToString()
        {
            String res = "";
            for (int i = 0; i < bddVar.Count; i++)
            {
                res += bddVar[i] + ", ";
            }
            return (String.Format("<LayoutVariable> type index: {0} \nBDD variables: {1}", typeIndex, res));
        }
    }
}

