/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
        Based on the CLab source code developed by Rune Mller Jensen.
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using buddy_sharp;

namespace CLab.BDD
{
    ///<summary>
    ///Class for BDD representation of the configuration space of a CP expression.
    ///</summary>
    public class Bval
    {
        /// <summary>
        /// Mapping from BDD variable assignments to expression values in 2-complement format
        /// </summary>
        private List<Bdd> bList;
        /// <summary>
        /// Constraint ensuring sub-operations are defined (e.g., no mod or div by zero)
        /// </summary>
        private Bdd defConBDD;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Bval"/> class.
        /// </summary>
        /// <param name="bddList">The list of BDDs</param>
        /// <param name="defcon">The default configuration.</param>
        public Bval(List<Bdd> bddList, Bdd defcon)
        {
            this.bList = bddList;
            this.defConBDD = defcon;
        }

        /// <summary>
        /// Gets or sets the BDD list.
        /// </summary>
        /// <value>The BDD list.</value>
        public List<Bdd> BDDList
        {
            get { return bList; }
            set { bList = value; }
        }

        /// <summary>
        /// Gets or sets the default configuration.
        /// </summary>
        /// <value>The default configuration.</value>
        public Bdd DefCon
        {
            get { return defConBDD; }
            set { defConBDD = value; }
        }
    }
}
