/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors". 
        Based on the CLab source code developed by Rune Mller Jensen.
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using buddy_sharp;

namespace CLab.BDD
{
    /// <summary>
    /// Priority queue node with IComparable implementation
    /// </summary>
    public class PQbdd : IComparable
    {
        private Bdd b;
        private int size;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:PQbdd"/> class.
        /// </summary>
        /// <param name="bc">The BDD.</param>
        public PQbdd(Bdd bc) 
        {
            this.b = bc;
            this.size = Bdd.NodeCount(this.b);
        }

        /// <summary>
        /// Gets the size.
        /// </summary>
       public int Size
        {
            get { return this.size; }
        }

        /// <summary>
        /// Gets the BDD.
        /// </summary>
        public Bdd BDD
        {
            get { return this.b; }
        }

        /// <summary>
        /// IComparable implementation. Compares PQbdds based on their size
        /// </summary>
        /// <param name="obj">PQbdd object</param>
        /// <returns>Result</returns>
        /// <exception cref="ArgumentException"/>
        public int CompareTo(object obj)
        {
            if (obj is PQbdd)
            {
                PQbdd temp = (PQbdd)obj;
                return this.size.CompareTo(temp.Size);
            }
            throw new ArgumentException("Comparison object is not a PQbdd");
        }
    }
}
