using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using CLab.Exceptions;

namespace CLab
{
    /// <summary>
    /// The CP object contains the internal data representation, parsed from the input XML file
    /// </summary>
    public class CP
    {
        private List<Type> types = new List<Type>();
        private List<Variable> variables = new List<Variable>();
        private List<Expression> rules = new List<Expression>();
        private String description;
        private String author;
        private String date;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="XmlFilename">Path to the XML file to parse input from</param>
        public CP(String XmlFilename)
        {
            ClabXmlParser parser = new ClabXmlParser();
            parser.ParseURL(XmlFilename, this);
        }

        /// <summary>
        /// Optional header information in the XML file.
        /// Description of the XML data (optional metadata)
        /// </summary>
        public String Description
        {
            get { return description; }
            set { description = value; }
        }

        /// <summary>
        /// Optional header information in the XML file.
        /// Author of the XML data (optional metadata)
        /// </summary>
        public String Author
        {
            get {return author; }
            set {author = value; }
        }

        /// <summary>
        /// Optional header information in the XML file.
        /// Timestamp of the XML data (optional metadata)
        /// </summary>
        public String Date 
        {
            get { return date; }
            set { date = value; }
        }

        /// <returns>List of type objects in the internal data representation</returns>
        public List<Type> Types
        {
            get { return types; }
        }

        /// <returns>List of variable objects in the internal data representation</returns>
        public List<Variable> Variables
        {
            get { return variables; }
        }

        /// <returns>List of rule (expression) objects in the internal data representation</returns>
        public List<Expression> Rules
        {
            get { return rules; }
        }

        /// <summary>
        /// Adds a type object to the internal data representation
        /// </summary>
        /// <param name="newType">The type object to add</param>
        public void AddType(Type newType)
        {
            types.Add(newType);
        }

        /// <summary>
        /// Adds a variable object to the internal data representation
        /// </summary>
        /// <param name="newVariable">The variable object to add</param>
        public void AddVariable(Variable newVariable)
        {
            variables.Add(newVariable);
        }

        /// <summary>
        /// Adds a rule (expression) object to the internal data representation
        /// </summary>
        /// <param name="newRule">The rule to add</param>
        public void AddRule(Expression newRule)
        {
            rules.Add(newRule);
        }
    }
}
