/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

using CLab.CSP;
using CLab.Exceptions;

using Casper.Data;

namespace CLab.Data
{
    /// <summary>
    /// An expression consisting of a left and right side and an operator.
    /// Implements the abstract Expression class.
    /// </summary>
    public class ExpressionBinary:Expression
    {
        private Expression right;
        private Expression left;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ExpressionBinary"/> class.
        /// </summary>
        /// <param name="left">The left expression.</param>
        /// <param name="type">The operator.</param>
        /// <param name="right">The right expression.</param>
        public ExpressionBinary(Expression left, Common.ExprType type, Expression right)
            : base(type)
        {
            this.left = left;
            this.right = right;

        }

        /// <summary>
        /// Returns left side expression.
        /// </summary>
        internal Expression Left
        {
            get { return left; }
        }
        /// <summary>
        /// Returns right side expression
        /// </summary>
        internal Expression Right
        {
            get { return right; }
        }

        /// <summary>
        /// Abstract method which runs CSPBuildExpr in CLab.CSP.CSPExpressionBuilder
        /// Double dispatch pattern
        /// </summary>
        /// <param name="builder">The CSPExpressionBuilder.</param>
        /// <returns>CSP-Expression</returns>
        /// <exception cref="CLab.Exceptions.ClabInternalErrorException"/>
        internal override CSPExpr RunCSPBuildExpr(CSPExpressionBuilder builder)
        {
            try
            {
                return builder.BuildCSPExpr(this);
            }
            catch (ClabInternalErrorException) { throw; }
        }

        /// <summary>
        /// Returns a textual description of the expression
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return (String.Format("({0} {1} {2})", left, Type2oper(base.Type), right));
        }
        
    }
}
