/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

using CLab.CSP;

using Casper.Data;

namespace CLab.Data
{
    /// <summary>
    /// Expression consisting of an integer. Implements the abstract
    /// Expression class.
    /// </summary>
    public class ExpressionInt : Expression
    {
        private int? integer;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:ExpressionInt"/> class.
        /// </summary>
        /// <param name="integer">The integer value of this expression.</param>
        public ExpressionInt(int integer)
            : base(Common.ExprType.et_val)
        {
            this.integer = integer;
        }


        /// <summary>
        /// Returns the integer value of this expression.
        /// </summary>
        public int IntegerValue
        {
            get { return (int)this.integer; }
        }


        /// <summary>
        /// Abstract method which runs CSPBuildExpr in CLab.CSP.CSPExpressionBuilder
        /// Double dispatch pattern
        /// </summary>
        /// <param name="builder">The CSPExpressionBuilder.</param>
        /// <returns>CSP Expression int</returns>
        internal override CSPExpr RunCSPBuildExpr(CSPExpressionBuilder builder)
        {
            return builder.BuildCSPExpr(this);
        }


        /// <summary>
        /// Returns a textual representation of the expression.
        /// </summary>
        public override string ToString()
        {
            return (String.Format("{0}", IntegerValue));
          
        }        
    }
}
