/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

using CLab.Exceptions;
using CLab.CSP;

using Casper.Data;


namespace CLab.Data
{
    /// <summary>
    /// An negated expression consisting of a left expression and negation type
    /// Implements the abstract Expression class.
    /// </summary>
    public class ExpressionNotNeg : Expression
    {
        private Expression left;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ExpressionNotNeg"/> class.
        /// </summary>
        /// <param name="type">The operator.</param>
        /// <param name="left">The left expression.</param>
        public ExpressionNotNeg(Common.ExprType type, Expression left)
            : base(type)
        {
            this.left = left;
        }


        /// <summary>
        /// Gets the left expression.
        /// </summary>
        internal Expression Left
        {
            get { return left; }
        }


        /// <summary>
        /// Abstract method which runs CSPBuildExpr in CLab.CSP.CSPExpressionBuilder
        /// Double dispatch pattern
        /// </summary>
        /// <param name="builder">The CSPExpressionBuilder.</param>
        /// <returns>Not-neg expression.</returns>
        /// <exception cref="CLab.Exceptions.ClabInternalErrorException"/>
        internal override CSPExpr RunCSPBuildExpr(CSPExpressionBuilder builder)
        {
            try
            {
                return builder.BuildCSPExpr(this);
            }
            catch (ClabInternalErrorException) { throw; }
        }

        /// <summary>
        /// Returns a textual representation of the not-neg-expression
        /// </summary>
        public override string ToString()
        {
            if (base.Type.Equals(Common.ExprType.et_neg)) return (String.Format("(-{0})", left));
            else return (String.Format("(!{0})", left));
        }

    }
}
