﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

namespace CLab.Data
{
    /// <summary>
    /// The Type class represents a CLab type
    /// TypeRange represents a range type. 
    /// Implements the abstract Type class.
    /// </summary>
    public class TypeRange : Type
    {
        private int startOfRange;
        private int endOfRange;


        /// <summary>
        /// Initializes a new instance of the <see cref="T:TypeRange"/> class.
        /// </summary>
        /// <param name="typeName">Name of the type.</param>
        /// <param name="startOfRange">The start of the range.</param>
        /// <param name="endOfRange">The end of the range.</param>
        public TypeRange(String typeName, int startOfRange, int endOfRange) : base(typeName)
        {
            this.startOfRange = startOfRange;
            this.endOfRange = endOfRange;
        }


        /// <summary>
        /// Gets or sets the integer value of the start of the range.
        /// </summary>
        /// <value>The integer value of the start of the range.</value>
        public int StartOfRange
        {
            get { return startOfRange; }
            set { startOfRange = value; }
        }

        /// <summary>
        /// Gets or sets the integer value of the end of the range.
        /// </summary>
        /// <value>The integer value of the end of the range.</value>
        public int EndOfRange
        {
            get { return endOfRange; }
            set { endOfRange = value; }
        }

        /// <summary>
        /// Returns the name of the type and its range.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            String range = "[" + this.startOfRange + " .. " + this.endOfRange + "]";
            return (String.Format("{0} {1};", TypeName, range));
        }


    }
}
