/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace CLab.Data
{
    /// <summary>
    /// The Variable class represents a CLab variable
    /// In addition to the types set by Type objects, CLab has an internal boolean type named bool
    /// </summary>
    public class Variable
    {
        private String typeName;
        private String variableName;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:Variable"/> class.
        /// </summary>
        /// <param name="typeName">Name of the type to use for this variable.</param>
        /// <param name="variableName">Name of the variable.</param>
        public Variable(String typeName, String variableName)
        {
            this.typeName = typeName;
            this.variableName = variableName;
        }

        /// <summary>
        /// Gets or sets the name of the variable type.
        /// </summary>
        /// <value>The name of the variable type.</value>
        public String TypeName
        {
            get { return typeName; }
            set { typeName = value; }
        }

        /// <summary>
        /// Gets or sets the name of the variable.
        /// </summary>
        /// <value>The name of the variable.</value>
        public String VariableName
        {
            get { return variableName; }
            set { variableName = value; }
        }

        /// <summary>
        /// Returns a string representation of the name of the variable type and the variable name.
        /// </summary>
        /// <returns>
        /// String representation of the name of the variable type and the variable name.
        /// </returns>
        public override string ToString()
        {
            return (String.Format("{0} {1};", typeName, variableName));
        }
    }
}
