/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.IO;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.Xml;
using com.calitha.goldparser.lalr;
using com.calitha.commons;

using CLab.Exceptions;

namespace com.calitha.goldparser
{
    /// <summary>
    /// Internal exception class used by the parser engine
    /// </summary>
    [Serializable()]
    public class SymbolException : System.Exception
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="T:SymbolException"/> class.
        /// </summary>
        /// <param name="message">The exception message which describes the reason for the exception.</param>
        public SymbolException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SymbolException"/> class.
        /// </summary>
        /// <param name="message">The exception message which describes the reason for the exception.</param>
        /// <param name="inner">The inner exception.</param>
        public SymbolException(string message,
            Exception inner)
            : base(message, inner)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:SymbolException"/> class.
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"></see> that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"></see> that contains contextual information about the source or destination.</param>
        /// <exception cref="T:System.Runtime.Serialization.SerializationException">The class name is null or <see cref="P:System.Exception.HResult"></see> is zero (0). </exception>
        /// <exception cref="T:System.ArgumentNullException">The info parameter is null. </exception>
        protected SymbolException(SerializationInfo info,
            StreamingContext context)
            : base(info, context)
        {
        }

    }

    /// <summary>
    /// Internal exception class used by the parser engine
    /// </summary>
    [Serializable()]
    public class RuleException : System.Exception
    {

        /// <summary>
        /// Initializes a new instance of the <see cref="T:RuleException"/> class.
        /// </summary>
        /// <param name="message">The exception message which describes the reason for the exception.</param>
        public RuleException(string message)
            : base(message)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:RuleException"/> class.
        /// </summary>
        /// <param name="message">The exception message which describes the reason for the exception.</param>
        /// <param name="inner">The inner exception.</param>
        public RuleException(string message,
                             Exception inner)
            : base(message, inner)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:RuleException"/> class.
        /// </summary>
        /// <param name="info">The <see cref="T:System.Runtime.Serialization.SerializationInfo"></see> that holds the serialized object data about the exception being thrown.</param>
        /// <param name="context">The <see cref="T:System.Runtime.Serialization.StreamingContext"></see> that contains contextual information about the source or destination.</param>
        /// <exception cref="T:System.Runtime.Serialization.SerializationException">The class name is null or <see cref="P:System.Exception.HResult"></see> is zero (0). </exception>
        /// <exception cref="T:System.ArgumentNullException">The info parameter is null. </exception>
        protected RuleException(SerializationInfo info,
                                StreamingContext context)
            : base(info, context)
        {
        }

    }

    enum SymbolConstants : int
    {
        SYMBOL_EOF = 0, // (EOF)
        SYMBOL_ERROR = 1, // (Error)
        SYMBOL_WHITESPACE = 2, // (Whitespace)
        SYMBOL_COMMENTEND = 3, // (Comment End)
        SYMBOL_COMMENTLINE = 4, // (Comment Line)
        SYMBOL_COMMENTSTART = 5, // (Comment Start)
        SYMBOL_MINUS = 6, // '-'
        SYMBOL_EXCLAM = 7, // '!'
        SYMBOL_PERCENT = 8, // '%'
        SYMBOL_LPARAN = 9, // '('
        SYMBOL_RPARAN = 10, // ')'
        SYMBOL_TIMES = 11, // '*'
        SYMBOL_COMMA = 12, // ','
        SYMBOL_DOT = 13, // '.'
        SYMBOL_DIV = 14, // '/'
        SYMBOL_SEMI = 15, // ';'
        SYMBOL_LBRACKET = 16, // '['
        SYMBOL_RBRACKET = 17, // ']'
        SYMBOL_LBRACE = 18, // '{'
        SYMBOL_RBRACE = 19, // '}'
        SYMBOL_PLUS = 20, // '+'
        SYMBOL_LT = 21, // '<'
        SYMBOL_GT = 22, // '>'
        SYMBOL_AND = 23, // AND
        SYMBOL_BOOL = 24, // BOOL
        SYMBOL_EQ = 25, // EQ
        SYMBOL_GTE = 26, // GTE
        SYMBOL_ID = 27, // ID
        SYMBOL_IMPL = 28, // IMPL
        SYMBOL_LTE = 29, // LTE
        SYMBOL_NE = 30, // NE
        SYMBOL_NUMBER = 31, // NUMBER
        SYMBOL_OR = 32, // OR
        SYMBOL_RULE = 33, // RULE
        SYMBOL_STRING = 34, // STRING
        SYMBOL_TYPE = 35, // TYPE
        SYMBOL_VARIABLE = 36, // VARIABLE
        SYMBOL_CP = 37, // <cp>
        SYMBOL_EXP = 38, // <exp>
        SYMBOL_EXP2 = 39, // <exp 2>
        SYMBOL_EXP3 = 40, // <exp 3>
        SYMBOL_EXP4 = 41, // <exp 4>
        SYMBOL_EXP5 = 42, // <exp 5>
        SYMBOL_EXP6 = 43, // <exp 6>
        SYMBOL_EXP7 = 44, // <exp 7>
        SYMBOL_EXP8 = 45, // <exp 8>
        SYMBOL_EXP9 = 46, // <exp 9>
        SYMBOL_ID2 = 47, // <id>
        SYMBOL_IDLST = 48, // <idlst>
        SYMBOL_INTEGER = 49, // <integer>
        SYMBOL_NUMBER2 = 50, // <number>
        SYMBOL_RULEDECL = 51, // <ruledecl>
        SYMBOL_RULEDECLS = 52, // <ruledecls>
        SYMBOL_TYPEDECL = 53, // <typedecl>
        SYMBOL_TYPEDECLS = 54, // <typedecls>
        SYMBOL_VARDECL = 55, // <vardecl>
        SYMBOL_VARDECLS = 56, // <vardecls>
        SYMBOL_VARTYPE = 57   // <vartype>

    };

    enum RuleConstants : int
    {
        RULE_CP_TYPE_VARIABLE_RULE = 0, // <cp> ::= TYPE <typedecls> VARIABLE <vardecls> RULE <ruledecls>
        RULE_CP_VARIABLE_RULE = 1, // <cp> ::= VARIABLE <vardecls> RULE <ruledecls>
        RULE_TYPEDECLS = 2, // <typedecls> ::= 
        RULE_TYPEDECLS2 = 3, // <typedecls> ::= <typedecl> <typedecls>
        RULE_TYPEDECL_LBRACKET_DOT_DOT_RBRACKET_SEMI = 4, // <typedecl> ::= <id> '[' <integer> '.' '.' <integer> ']' ';'
        RULE_TYPEDECL_LBRACE_RBRACE_SEMI = 5, // <typedecl> ::= <id> '{' <idlst> '}' ';'
        RULE_VARDECLS = 6, // <vardecls> ::= 
        RULE_VARDECLS2 = 7, // <vardecls> ::= <vardecl> <vardecls>
        RULE_VARDECL_SEMI = 8, // <vardecl> ::= <vartype> <idlst> ';'
        RULE_VARTYPE_BOOL = 9, // <vartype> ::= BOOL
        RULE_VARTYPE = 10, // <vartype> ::= <id>
        RULE_IDLST = 11, // <idlst> ::= <id>
        RULE_IDLST_COMMA = 12, // <idlst> ::= <id> ',' <idlst>
        RULE_RULEDECLS = 13, // <ruledecls> ::= 
        RULE_RULEDECLS2 = 14, // <ruledecls> ::= <ruledecl> <ruledecls>
        RULE_RULEDECL_SEMI = 15, // <ruledecl> ::= <exp> ';'
        RULE_EXP_OR = 16, // <exp> ::= <exp> OR <exp 2>
        RULE_EXP = 17, // <exp> ::= <exp 2>
        RULE_EXP2_AND = 18, // <exp 2> ::= <exp 2> AND <exp 3>
        RULE_EXP2 = 19, // <exp 2> ::= <exp 3>
        RULE_EXP3_NE = 20, // <exp 3> ::= <exp 3> NE <exp 4>
        RULE_EXP3_EQ = 21, // <exp 3> ::= <exp 3> EQ <exp 4>
        RULE_EXP3 = 22, // <exp 3> ::= <exp 4>
        RULE_EXP4_LTE = 23, // <exp 4> ::= <exp 4> LTE <exp 5>
        RULE_EXP4_GTE = 24, // <exp 4> ::= <exp 4> GTE <exp 5>
        RULE_EXP4_LT = 25, // <exp 4> ::= <exp 4> '<' <exp 5>
        RULE_EXP4_GT = 26, // <exp 4> ::= <exp 4> '>' <exp 5>
        RULE_EXP4 = 27, // <exp 4> ::= <exp 5>
        RULE_EXP5_IMPL = 28, // <exp 5> ::= <exp 5> IMPL <exp 6>
        RULE_EXP5 = 29, // <exp 5> ::= <exp 6>
        RULE_EXP6_MINUS = 30, // <exp 6> ::= <exp 6> '-' <exp 7>
        RULE_EXP6_PLUS = 31, // <exp 6> ::= <exp 6> '+' <exp 7>
        RULE_EXP6 = 32, // <exp 6> ::= <exp 7>
        RULE_EXP7_PERCENT = 33, // <exp 7> ::= <exp 7> '%' <exp 8>
        RULE_EXP7_DIV = 34, // <exp 7> ::= <exp 7> '/' <exp 8>
        RULE_EXP7_TIMES = 35, // <exp 7> ::= <exp 7> '*' <exp 8>
        RULE_EXP7 = 36, // <exp 7> ::= <exp 8>
        RULE_EXP8_MINUS = 37, // <exp 8> ::= '-' <exp 9>
        RULE_EXP8_EXCLAM = 38, // <exp 8> ::= '!' <exp 9>
        RULE_EXP8 = 39, // <exp 8> ::= <exp 9>
        RULE_EXP9 = 40, // <exp 9> ::= <number>
        RULE_EXP92 = 41, // <exp 9> ::= <id>
        RULE_EXP9_LPARAN_RPARAN = 42, // <exp 9> ::= '(' <exp> ')'
        RULE_ID_ID = 43, // <id> ::= ID
        RULE_ID_STRING = 44, // <id> ::= STRING
        RULE_INTEGER = 45, // <integer> ::= <number>
        RULE_INTEGER_MINUS = 46, // <integer> ::= '-' <number>
        RULE_NUMBER_NUMBER = 47   // <number> ::= NUMBER

    };

    /// <summary>
    /// Token based parser using the GOLD Parsing system.
    /// Grammar tables for parsing are compiled from the parser grammar located in the doc folder.
    /// The compiled grammar is located in the Auxiliary folder and is integrated in the compiled binaries.
    /// </summary>
    /// <remarks>
    /// This source file was initially generated by the GOLD Parsing system, based on grammar definition.
    /// Not all placeholder code has been replaced/removed to facilitate further use.
    /// </remarks>
    /// <seealso href="http://www.devincook.com/goldparser/">GOLD Parsing system</seealso>
    public class TokenParser
    {
        private LALRParser parser;
        private List<Object> results = new List<Object>();
        private XmlWriter xWriter;
        private bool holdForNegation = false;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:TokenParser"/> class. Parsing from a file defined by <paramref name="filename"/>.
        /// </summary>
        /// <param name="filename">Filename of input file for parsing.</param>
        /// <param name="xw">The XML writer to write output to.</param>
        public TokenParser(string filename, XmlWriter xw)
        {
            xWriter = xw;
            FileStream stream = new FileStream(filename,
                                               FileMode.Open,
                                               FileAccess.Read,
                                               FileShare.Read);
            Init(stream);
            stream.Close();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:TokenParser"/> class.
        /// </summary>
        /// <param name="stream">The stream to parse.</param>
        /// <param name="xw">The XML writer to write output to.</param>
        public TokenParser(Stream stream, XmlWriter xw)
        {
            xWriter = xw;
            Init(stream);
        }


        /// <summary>
        /// Inits the specified stream.
        /// </summary>
        /// <param name="stream">The stream.</param>
        private void Init(Stream stream)
        {
            CGTReader reader = new CGTReader(stream);
            parser = reader.CreateNewParser();
            parser.TrimReductions = false;
            parser.StoreTokens = LALRParser.StoreTokensMode.NoUserObject;

            parser.OnTokenError += new LALRParser.TokenErrorHandler(TokenErrorEvent);
            parser.OnParseError += new LALRParser.ParseErrorHandler(ParseErrorEvent);
        }

        /// <summary>
        /// Parses the specified source.
        /// </summary>
        /// <param name="source">The source in string representation.</param>
        public void Parse(string source)
        {
            NonterminalToken token = parser.Parse(source);
            if (token != null)
            {
                Object obj = CreateObject(token);
            }
        }

        /// <summary>
        /// Main object creatition method. All tokens are either terminal or nonterminal.
        /// </summary>
        /// <param name="token"></param>
        /// <returns></returns>
        private Object CreateObject(Token token)
        {
            if (token is TerminalToken)
                return CreateObjectFromTerminal((TerminalToken)token);
            else
                return CreateObjectFromNonterminal((NonterminalToken)token);
        }

        /// <summary>
        /// Manage a terminal token object
        /// </summary>
        /// <param name="token">Token to manage</param>
        /// <returns>String or int representation of token, or null</returns>
        /// <exception cref="SymbolException"/>
        private Object CreateObjectFromTerminal(TerminalToken token)
        {
            switch (token.Symbol.Id)
            {
                case (int)SymbolConstants.SYMBOL_EOF: //(EOF)
                case (int)SymbolConstants.SYMBOL_ERROR: //(Error)
                case (int)SymbolConstants.SYMBOL_WHITESPACE: //(Whitespace)
                case (int)SymbolConstants.SYMBOL_COMMENTEND: //(Comment End)
                case (int)SymbolConstants.SYMBOL_COMMENTLINE: //(Comment Line)
                case (int)SymbolConstants.SYMBOL_COMMENTSTART: //(Comment Start)
                    return null;

                //Operators and IDs
                case (int)SymbolConstants.SYMBOL_MINUS:
                case (int)SymbolConstants.SYMBOL_EXCLAM:
                case (int)SymbolConstants.SYMBOL_PERCENT:
                case (int)SymbolConstants.SYMBOL_TIMES:
                case (int)SymbolConstants.SYMBOL_DIV:
                case (int)SymbolConstants.SYMBOL_PLUS:
                case (int)SymbolConstants.SYMBOL_LT:
                case (int)SymbolConstants.SYMBOL_GT:
                case (int)SymbolConstants.SYMBOL_BOOL:
                case (int)SymbolConstants.SYMBOL_EQ:
                case (int)SymbolConstants.SYMBOL_GTE:
                case (int)SymbolConstants.SYMBOL_ID:
                case (int)SymbolConstants.SYMBOL_STRING:
                case (int)SymbolConstants.SYMBOL_IMPL:
                case (int)SymbolConstants.SYMBOL_LTE:
                case (int)SymbolConstants.SYMBOL_NE:
                    return token.Text;

                case (int)SymbolConstants.SYMBOL_AND:
                case (int)SymbolConstants.SYMBOL_OR:
                    return token.Text.Substring(0, 1);

                //Integers
                case (int)SymbolConstants.SYMBOL_NUMBER:
                case (int)SymbolConstants.SYMBOL_NUMBER2:
                case (int)SymbolConstants.SYMBOL_INTEGER:
                    return int.Parse(token.Text);

                //Terminal objects which we do not need return types
                case (int)SymbolConstants.SYMBOL_LPARAN:
                case (int)SymbolConstants.SYMBOL_RPARAN:
                case (int)SymbolConstants.SYMBOL_COMMA:
                case (int)SymbolConstants.SYMBOL_DOT:
                case (int)SymbolConstants.SYMBOL_SEMI:
                case (int)SymbolConstants.SYMBOL_LBRACKET:
                case (int)SymbolConstants.SYMBOL_RBRACKET:
                case (int)SymbolConstants.SYMBOL_LBRACE:
                case (int)SymbolConstants.SYMBOL_RBRACE:
                case (int)SymbolConstants.SYMBOL_RULE:
                case (int)SymbolConstants.SYMBOL_TYPE:
                case (int)SymbolConstants.SYMBOL_VARIABLE:
                case (int)SymbolConstants.SYMBOL_CP:
                case (int)SymbolConstants.SYMBOL_EXP:
                case (int)SymbolConstants.SYMBOL_EXP2:
                case (int)SymbolConstants.SYMBOL_EXP3:
                case (int)SymbolConstants.SYMBOL_EXP4:
                case (int)SymbolConstants.SYMBOL_EXP5:
                case (int)SymbolConstants.SYMBOL_EXP6:
                case (int)SymbolConstants.SYMBOL_EXP7:
                case (int)SymbolConstants.SYMBOL_EXP8:
                case (int)SymbolConstants.SYMBOL_EXP9:
                case (int)SymbolConstants.SYMBOL_ID2:
                case (int)SymbolConstants.SYMBOL_IDLST:
                case (int)SymbolConstants.SYMBOL_RULEDECL:
                case (int)SymbolConstants.SYMBOL_RULEDECLS:
                case (int)SymbolConstants.SYMBOL_TYPEDECL:
                case (int)SymbolConstants.SYMBOL_TYPEDECLS:
                case (int)SymbolConstants.SYMBOL_VARDECL:
                case (int)SymbolConstants.SYMBOL_VARDECLS:
                case (int)SymbolConstants.SYMBOL_VARTYPE:
                    return null;
            }
            throw new SymbolException("Unknown symbol");
        }

        /// <summary>
        /// Manage a nonterminal token object
        /// </summary>
        /// <param name="token">Token to manage</param>
        /// <returns>Internal token</returns>
        /// <exception cref="ClabException"/>
        /// <exception cref="RuleException"/>
        public Object CreateObjectFromNonterminal(NonterminalToken token)
        {
            switch (token.Rule.Id)
            {
                case (int)RuleConstants.RULE_CP_TYPE_VARIABLE_RULE:
                    //<cp> ::= TYPE <typedecls> VARIABLE <vardecls> RULE <ruledecls>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            NonterminalToken t2 = (NonterminalToken)t;
                            if ((t2.Rule.Id == (int)RuleConstants.RULE_TYPEDECLS) | (t2.Rule.Id == (int)RuleConstants.RULE_TYPEDECLS2))
                            {
                                xWriter.WriteStartElement("type");
                                CreateObjectFromNonterminal(t2);
                                xWriter.WriteEndElement();
                            }
                            else if ((t2.Rule.Id == (int)RuleConstants.RULE_VARDECLS) | (t2.Rule.Id == (int)RuleConstants.RULE_VARDECLS2))
                            {
                                xWriter.WriteStartElement("variable");
                                CreateObjectFromNonterminal(t2);
                                xWriter.WriteEndElement();
                            }
                            else if ((t2.Rule.Id == (int)RuleConstants.RULE_RULEDECLS) | (t2.Rule.Id == (int)RuleConstants.RULE_RULEDECLS2))
                            {
                                xWriter.WriteStartElement("rule");
                                CreateObjectFromNonterminal(t2);
                                xWriter.WriteEndElement();
                            }
                            else throw new TokenParserException("Error parsing text to XML. Wrong type: " + t2.Rule.Id);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_CP_VARIABLE_RULE:
                    //<cp> ::= VARIABLE <vardecls> RULE <ruledecls>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_TYPEDECLS:
                    //<typedecls> ::= 
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_TYPEDECLS2:
                    //<typedecls> ::= <typedecl> <typedecls>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_TYPEDECL_LBRACKET_DOT_DOT_RBRACKET_SEMI:
                    //<typedecl> ::= <id> '[' <integer> '.' '.' <integer> ']' ';'
                    int k = 0;
                    for (int j = 0; j < token.Tokens.Length; j++)
                    {
                        Token t = token.Tokens[j];
                        if (t is NonterminalToken)
                        {
                            switch (k)
                            {
                                case 0:
                                    xWriter.WriteStartElement("typeDecl");
                                    xWriter.WriteAttributeString("name", (String)CreateObjectFromNonterminal((NonterminalToken)t));
                                    break;
                                case 1:
                                    xWriter.WriteStartElement("range");
                                    int startInt = (int)CreateObjectFromNonterminal((NonterminalToken)t);
                                    xWriter.WriteAttributeString("start", startInt.ToString());
                                    break;
                                case 2:
                                    int endInt = (int)CreateObjectFromNonterminal((NonterminalToken)t);
                                    xWriter.WriteAttributeString("end", endInt.ToString());
                                    xWriter.WriteEndElement();
                                    xWriter.WriteEndElement();
                                    break;
                                default:
                                    break;
                            }
                            k++;
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_TYPEDECL_LBRACE_RBRACE_SEMI:
                    //<typedecl> ::= <id> '{' <idlst> '}' ';'
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            TerminalToken t2 = (TerminalToken)t;
                            if (t2.Text.Equals(";"))
                            {
                                xWriter.WriteEndElement(); //</typeDecl>
                                results.Clear();
                            }
                        }
                        else
                        {
                            NonterminalToken t2 = (NonterminalToken)t;
                            if (t2.Rule.Id == (int)RuleConstants.RULE_ID_ID)
                            {
                                xWriter.WriteStartElement("typeDecl");
                                xWriter.WriteAttributeString("name", (String)CreateObjectFromNonterminal(t2));
                                results.Clear();
                            }
                            else if ((t2.Rule.Id == (int)RuleConstants.RULE_IDLST) | (t2.Rule.Id == (int)RuleConstants.RULE_IDLST_COMMA))
                            {
                                CreateObjectFromNonterminal(t2);
                                for (int i = 0; i < results.Count; i++)
                                {
                                    String aResult = (String)results[i];
                                    if (aResult.StartsWith("\"")) aResult = aResult.Substring(1, aResult.Length - 2);

                                    xWriter.WriteStartElement("typeVar");
                                    xWriter.WriteString(aResult);
                                    xWriter.WriteEndElement();
                                }
                            }
                            else if (t2.Rule.Id == (int)RuleConstants.RULE_ID_STRING)
                            {
                                throw new TokenParserException("Type name is not allowed to be a quoted string (" + t2.ToString() + ")");
                            }
                            else throw new TokenParserException("Error parsing text to XML. Wrong type: " + t.GetType());
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_VARDECLS:
                case (int)RuleConstants.RULE_VARDECLS2:
                    //<vardecls> ::=
                    //<vardecls> ::= <vardecl> <vardecls>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_VARDECL_SEMI:
                    //<vardecl> ::= <vartype> <idlst> ';'
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                            xWriter.WriteEndElement(); //</varDecl>
                        }
                        else
                        {
                            NonterminalToken t2 = (NonterminalToken)t;
                            if ((t2.Rule.Id == (int)RuleConstants.RULE_VARTYPE) | (t2.Rule.Id == (int)RuleConstants.RULE_VARTYPE_BOOL))
                            {
                                xWriter.WriteStartElement("varDecl");
                                xWriter.WriteAttributeString("varType", (String)CreateObjectFromNonterminal(t2));
                            }
                            else if (t2.Rule.Id == (int)RuleConstants.RULE_IDLST)
                            {
                                xWriter.WriteStartElement("varName");
                                String varNameString = (String)CreateObjectFromNonterminal(t2);
                                if (varNameString.Contains("\"")) throw new TokenParserException("Variable name is not allowed to be a quoted string (" + varNameString + ")");
                                xWriter.WriteString(varNameString);
                                xWriter.WriteEndElement();
                                results.Clear();
                            }
                            else if (t2.Rule.Id == (int) RuleConstants.RULE_IDLST_COMMA)
                            {
                                CreateObjectFromNonterminal(t2);
                                if (results.Count > 0)
                                {
                                    foreach (String s in results)
                                    {
                                        if (s.Contains("\"")) throw new TokenParserException("Variable name is not allowed to be a quoted string (" + s + ")");
                                        xWriter.WriteStartElement("varName");
                                        xWriter.WriteString(s);
                                        xWriter.WriteEndElement();
                                    }
                                }
                                results.Clear();
                            }
                            else throw new TokenParserException("Error parsing text to XML. Wrong type: " + t2.Rule.Id);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_VARTYPE_BOOL:
                    //<vartype> ::= BOOL
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                            return "bool";
                        }
                        else
                        {
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_VARTYPE:
                    //<vartype> ::= <id>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            NonterminalToken t2 = (NonterminalToken)t;
                            if (t2.Rule.Id == (int)RuleConstants.RULE_ID_STRING)
                                throw new TokenParserException("Variable type is not allowed to be a quoted string (" + t2.ToString() + ")");
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_IDLST:
                    //<idlst> ::= <id>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            results.Add(CreateObjectFromNonterminal((NonterminalToken)t));
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_IDLST_COMMA:
                    //<idlst> ::= <id> ',' <idlst>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            NonterminalToken t2 = (NonterminalToken)t;
                            if (t2.Rule.Id == (int)RuleConstants.RULE_ID_ID | t2.Rule.Id == (int)RuleConstants.RULE_ID_STRING)
                            {
                                results.Add(CreateObjectFromNonterminal(t2));
                            }
                            else if ((t2.Rule.Id == (int)RuleConstants.RULE_IDLST) | (t2.Rule.Id == (int)RuleConstants.RULE_IDLST_COMMA))
                            {
                                CreateObjectFromNonterminal(t2);
                            }
                            else throw new TokenParserException("Error parsing text to XML. Wrong type: " + t.GetType());
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_RULEDECLS:
                    //<ruledecls> ::= 
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_RULEDECLS2:
                    //<ruledecls> ::= <ruledecl> <ruledecls>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_RULEDECL_SEMI:
                    //<ruledecl> ::= <exp> ';'
                    xWriter.WriteStartElement("ruleDecl");
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    xWriter.WriteEndElement();//</ruleDecl>
                    return null;

                case (int)RuleConstants.RULE_EXP:
                case (int)RuleConstants.RULE_EXP2:
                case (int)RuleConstants.RULE_EXP3:
                case (int)RuleConstants.RULE_EXP4:
                case (int)RuleConstants.RULE_EXP5:
                case (int)RuleConstants.RULE_EXP6:
                case (int)RuleConstants.RULE_EXP7:
                case (int)RuleConstants.RULE_EXP8:
                case (int)RuleConstants.RULE_EXP9:
                    //<exp> ::= <exp>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            return CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_EXP_OR:
                case (int)RuleConstants.RULE_EXP2_AND:
                case (int)RuleConstants.RULE_EXP3_NE:
                case (int)RuleConstants.RULE_EXP3_EQ:
                case (int)RuleConstants.RULE_EXP4_LTE:
                case (int)RuleConstants.RULE_EXP4_GTE:
                case (int)RuleConstants.RULE_EXP4_LT:
                case (int)RuleConstants.RULE_EXP4_GT:
                case (int)RuleConstants.RULE_EXP5_IMPL:
                case (int)RuleConstants.RULE_EXP6_MINUS:
                case (int)RuleConstants.RULE_EXP6_PLUS:
                case (int)RuleConstants.RULE_EXP7_PERCENT:
                case (int)RuleConstants.RULE_EXP7_DIV:
                case (int)RuleConstants.RULE_EXP7_TIMES:
                    //<exp> ::= <exp> OPERATOR <exp>
                    if (holdForNegation) return null;

                    for (int j = 0; j < token.Tokens.Length; j++)
                    {
                        Token t = token.Tokens[j];
                        if (t is TerminalToken)
                        {
                            xWriter.WriteStartElement("operator");
                            xWriter.WriteString((String)CreateObjectFromTerminal((TerminalToken)t));
                            xWriter.WriteEndElement();
                        }
                        else
                        {
                            String side = "";
                            if (j == 0) side = "left";
                            else side = "right";

                            xWriter.WriteStartElement(side);
                            Object t2 = CreateObjectFromNonterminal((NonterminalToken)t);
                            if (t2 is System.String)
                            {
                                xWriter.WriteStartElement("id");
                                String anID = (String)t2;
                                if (anID.StartsWith("\"")) anID = anID.Substring(1, anID.Length - 2);
                                xWriter.WriteString(anID);
                                xWriter.WriteEndElement();
                            }
                            else if (t2 != null)
                            {
                                xWriter.WriteStartElement("integer");
                                xWriter.WriteString(t2.ToString());
                                xWriter.WriteEndElement();
                            }
                            xWriter.WriteEndElement(); //</left> | </right>
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_EXP8_MINUS:
                    //<exp> ::= '-' <exp>
                    //<exp> ::= '-' <number>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            holdForNegation = true;
                            Object t2 = CreateObjectFromNonterminal((NonterminalToken)t);
                            holdForNegation = false;
                            if (t2 is System.Int32)
                            {
                                int negInt = (int)t2 * -1;
                                xWriter.WriteStartElement("integer");
                                xWriter.WriteString(negInt.ToString());
                                xWriter.WriteEndElement();
                            }
                            else if (t2 is String)
                            {
                                xWriter.WriteStartElement("neg");
                                xWriter.WriteStartElement("id");
                                xWriter.WriteString(t2.ToString());
                                xWriter.WriteEndElement();
                                xWriter.WriteEndElement();
                            }
                            else
                            {
                                xWriter.WriteStartElement("neg");
                                CreateObjectFromNonterminal((NonterminalToken)t);
                                xWriter.WriteEndElement();
                            }
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_EXP8_EXCLAM:
                    //<exp> ::= '!' <exp>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            xWriter.WriteStartElement("not");
                            Object t2 = CreateObjectFromNonterminal((NonterminalToken)t);
                            if (t2 is System.String)
                            {
                                xWriter.WriteStartElement("id");
                                xWriter.WriteString((String)t2);
                                xWriter.WriteEndElement();
                            }
                            xWriter.WriteEndElement();
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_EXP92:
                    //<exp> ::= <id>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_EXP9_LPARAN_RPARAN:
                    //<exp> ::= '(' <exp> ')'
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_ID_ID:
                    //<id> ::= ID
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            return CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_ID_STRING:
                    //<id> ::= STRING
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            return CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_INTEGER:
                    //<integer> ::= <number>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            return CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_INTEGER_MINUS:
                    //<integer> ::= - <number>
                    foreach (Token t in token.Tokens)
                    {
                        if (t is NonterminalToken)
                        {
                            Object t2 = CreateObjectFromNonterminal((NonterminalToken)t);
                            if (t2 is System.Int32)
                            {
                                int negInt = (int)t2 * -1;
                                return negInt;
                            }
                        }
                    }
                    return null;

                case (int)RuleConstants.RULE_NUMBER_NUMBER:
                    //<number> ::= NUMBER
                    foreach (Token t in token.Tokens)
                    {
                        if (t is TerminalToken)
                        {
                            return CreateObjectFromTerminal((TerminalToken)t);
                        }
                        else
                        {
                            return CreateObjectFromNonterminal((NonterminalToken)t);
                        }
                    }
                    return null;

            }
            throw new RuleException("Unknown rule");
        }

        /// <summary>
        /// Token error
        /// </summary>
        /// <param name="parser">The current parser.</param>
        /// <param name="args">The <see cref="T:com.calitha.goldparser.TokenErrorEventArgs"/> instance containing the event data.</param>
        private void TokenErrorEvent(LALRParser parser, TokenErrorEventArgs args)
        {
            throw new TokenParserException(String.Format("There was a problem saving the file\n\nToken error caused by token: {0}\n\nPosition: {1}, Line: {2}, Column: {3}", args.Token.Text, args.Token.Location.Position, args.Token.Location.LineNr, args.Token.Location.ColumnNr));
        }

        /// <summary>
        /// Parse error
        /// </summary>
        /// <param name="parser">The current parser.</param>
        /// <param name="args">The <see cref="T:com.calitha.goldparser.ParseErrorEventArgs"/> instance containing the event data.</param>
        private void ParseErrorEvent(LALRParser parser, ParseErrorEventArgs args)
        {
            String expectedTokens = args.ExpectedTokens.ToString();
            expectedTokens = expectedTokens.Replace("OR", "||");
            expectedTokens = expectedTokens.Replace("NE", "!=");
            expectedTokens = expectedTokens.Replace("LTE", "<=");
            expectedTokens = expectedTokens.Replace("IMPL", ">>");
            expectedTokens = expectedTokens.Replace("GTE", ">=");
            expectedTokens = expectedTokens.Replace("EQ", "==");
            expectedTokens = expectedTokens.Replace("AND", "&&");

            throw new TokenParserException(String.Format("There was a problem saving the file\n\nParse error caused by token: {0}\n\nExpected: {1}\n\nPosition: {2}, Line: {3}, Column: {4}",
                args.UnexpectedToken.Text,
                expectedTokens,
                args.UnexpectedToken.Location.Position,
                args.UnexpectedToken.Location.LineNr,
                args.UnexpectedToken.Location.ColumnNr));
        }
    }
}
