﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace CLab
{
    /// <summary>
    /// The Type class represents a CLab type
    /// TypeenumeratorsList represents an enumeration type with a list of valid states (domain)
    /// </summary>
    public class TypeEnumeration : Type
    {
        private List<String> enumeratorList;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="typeName">Name of the type</param>
        /// <param name="enumeratorsList">List of valid states for the type</param>
        public TypeEnumeration(String typeName, List<String> enumeratorsList)
            : base(typeName)
        {
            this.enumeratorList = enumeratorsList;
        }

        ///<returns>List of valid states for the type</returns>
        public List<String> EnumeratorsList
        {
            get { return enumeratorList; }
        }

        ///<returns>A string representation of the valid states for the type in the form {value1, value2, value3, ... , valueN}</returns>
        public String GetDomain
        {
            get
            {
                String domain = "{";
                for (int i = 0; i < this.enumeratorList.Count; i++)
                {
                    domain += this.enumeratorList[i];
                    if (i < this.enumeratorList.Count - 1)
                        domain += ", ";
                }
                domain += "}";
                return domain;
            }
        }

            public override String ToString()
        {
            String list = "{";
            for (int i = 0; i < this.enumeratorList.Count; i++)
            {
                list += this.enumeratorList[i];
                if (i < this.enumeratorList.Count - 1) list += ",";
            }
            list += "}";

            

            return (String.Format("<TypeEnumeration> {0} {1}, {2}", TypeName, list, base.ToString()));
        }

    }
}
