﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

namespace CLab
{
    /// <summary>
    /// Class with valid domain information for a paricular variable.
    /// </summary>
    public class ValidDomain
    {
        private String varName;
        private String typeName;
        private CPTypes typeFormat;
        private List<String> domains;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ValidDomain"/> class.
        /// </summary>
        /// <param name="varName">Name of the variable.</param>
        /// <param name="typeName">Name of the type.</param>
        /// <param name="typeFormat">The CPTypes type format.</param>
        /// <param name="domains">The list of domain values.</param>
        public ValidDomain(String varName, String typeName, CPTypes typeFormat, List<String> domains)
        {
            this.varName = varName;
            this.typeName = typeName;
            this.typeFormat = typeFormat;
            this.domains = domains;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ValidDomain"/> class.
        /// </summary>
        /// <param name="varName">Name of the variable.</param>
        /// <param name="typeName">Name of the type.</param>
        /// <param name="typeFormat">The CPTypes type format.</param>
        public ValidDomain(String varName, String typeName, CPTypes typeFormat)
        {
            this.varName = varName;
            this.typeName = typeName;
            this.typeFormat = typeFormat;
            domains = new List<String>();
        }

        /// <summary>
        /// Gets the name of the variable.
        /// </summary>
        public String VarName
        {
            get { return varName; }
        }

        /// <summary>
        /// Gets the valid domains.
        /// </summary>
        public List<String> Domains
        {
            get { return domains; }
        }

        /// <summary>
        /// Adds a valid domain.
        /// </summary>
        /// <param name="domain">The valid domain to add.</param>
        /// <returns></returns>
        internal ValidDomain AddValidDomain(String domain)
        {
            domains.Add(domain);
            return this;
        }

        /// <summary>
        /// Empties the valid domain list.
        /// </summary>
        internal void EmptyValidDomain()
        {
            domains.Clear();
        }
    }
}
