﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

namespace CLab
{
    /// <summary>
    /// Class with the valid domains of each variable. This class has a list
    /// of <see cref="T:ValidDomain"></see> objects for each variable. 
    /// This is the result representation returned out of the library.
    /// </summary>
    public class ValidDomains
    {
        List<ValidDomain> validDoms;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ValidDomains"/> class.
        /// </summary>
        /// <param name="validDoms">A list of valid domains.</param>
        public ValidDomains(List<ValidDomain> validDoms)
        {
            this.validDoms = validDoms;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ValidDomains"/> class.
        /// </summary>
        public ValidDomains()
        {
            validDoms = new List<ValidDomain>();
        }

        /// <summary>
        /// Adds the valid domain to the list.
        /// </summary>
        /// <param name="validDomain">The valid domain to add to the list.</param>
        /// <returns>The index of the last added valid domain.</returns>
        public int addValidDomain(ValidDomain validDomain)
        {
            validDoms.Add(validDomain);
            return validDoms.Count - 1;
        }

        /// <summary>
        /// Gets the list of the valid domains.
        /// </summary>
        public List<ValidDomain> ValidDoms
        {
            get { return validDoms; }
        }
    }
}