/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using Iesi.Collections;

namespace Casper.Data
{
    /// <summary>
    /// A class representing a list of adjacent nodes in a Constraint Graph.
    /// </summary>
    public class AdjacencyList
    {
        private int variableWidth; //the size of the adjacency list
        private Set variables; //Variable id's in this Set represents  nodes that are interconnected in the constraint graph

        /// <summary>
        /// Initializes a new instance of the <see cref="T:AdjacencyList"/> class.
        /// </summary>
        public AdjacencyList() 
        {
            this.variableWidth = 0;
            variables = new HashedSet();
        }

        /// <summary>
        /// Gets or sets the width of the variable.
        /// </summary>
        /// <value>The width of the variable.</value>
        public int VariableWidth
        {
            get { return variableWidth; }
            set { variableWidth = value; }
        }

        /// <summary>
        /// Gets or sets the list of adjacent nodes.
        /// </summary>
        /// <value>The A list.</value>
        public Set Variables
        {
            get { return variables; }
            set { variables = value; }
        }

        /// <summary>
        /// Adds a set of variable ids to the adjacency list.
        /// </summary>
        /// <param name="vars">The set of variables to add to the adjacency list.</param>
        public void AddAllVars(ISet vars)
        {
            variables = (Set)variables.Union(vars);
            UpdateVarWidth();
        }

        /// <summary>
        /// Adds a single variable id to the adjacency list.
        /// </summary>
        /// <param name="varID">The variable id to add to the adjacency list.</param>
        public void AddVar(int varID)
        {
            if (!variables.Contains(varID))
            {
                Variables.Add(varID);
                UpdateVarWidth();
            }
        }

        /// <summary>
        /// Updates the width of the variable.
        /// </summary>
        public void UpdateVarWidth()
        {
            this.variableWidth = variables.Count;
        }

    }
}
