﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;

using Casper.Algorithm;

namespace Casper.Data
{
    /// <summary>
    /// A class for inverted expressions.
    /// The class consist of an expression which should be inverted.
    /// Implements the abstract CSPExpr class.
    /// </summary>
    public class CSPExprNot : CSPExpr
    {
        private CSPExpr left;
       
        /// <summary>
        /// Initializes a new instance of the <see cref="T:CSPExprNotNeg"/> class.
        /// </summary>
        /// <param name="left">The left expression</param>
        public CSPExprNot(CSPExpr left)
            :base()
        {
            this.left = left;
        }

        /// <summary>
        /// Gets the expression.
        /// </summary>
        /// <value>The left.</value>
        public CSPExpr Left
        {
            get { return left; }
        }

        /// <summary>
        /// Overriden method which runs <see cref="T:IConsistent.RunExpression"></see> 
        /// in the consistent implementation. Double dispatch pattern.
        /// </summary>
        /// <param name="consistentImpl"></param>
        /// <returns>An int, telling if the expression is valid or not.</returns>
        internal override int ConsistentCheckExpression(Consistent consistentImpl)
        {
            return consistentImpl.ConsistentCheckExpression(this);
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"></see> that represents the current <see cref="T:System.Object"></see>.
        /// </returns>
        public override string ToString()
        {
            return left.ToString();
        }
    }
}
