/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using Iesi.Collections;

namespace Casper.Data
{
    /// <summary>
    /// Stores wrapped expression objects along with a set of its variables IDs.
    /// Used during consistency to filter out expressions which have variables which isn't set yet.
    /// </summary>
    public class CSPExprWrapper
    {
        private CSPExpr ruleExpression;
        private Set variablesInExpr;

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="anExpr">An expression object representing a rule</param>
        /// <param name="variablesInExpr">Set of variable IDs used in the rule</param>
        public CSPExprWrapper(CSPExpr anExpr, Set variablesInExpr)
        {
            this.ruleExpression = anExpr;
            this.variablesInExpr = variablesInExpr;
        }

        /// <summary>
        /// The contained expression object
        /// </summary>
        public CSPExpr RuleExpression
        {
            get { return ruleExpression; }
            set { this.ruleExpression = value; }
        }

        /// <summary>
        /// Set of variables which the contained expression object consists of
        /// </summary>
        public Set VariablesInExpr
        {
            get { return variablesInExpr; }
            set { this.variablesInExpr = value; }
        }
    }
}
