﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.CompilerServices;

namespace Casper.Data
{
    /// <summary>
    /// A class which keeps track of the expressions for a CSP problem.
    /// </summary>
    public class CSPExpressions
    {
        private List<CSPExprWrapper> wrappedExprList;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CSPExpressions"/> class.
        /// </summary>
        public CSPExpressions()
        {
            wrappedExprList = new List<CSPExprWrapper>();
        }

        /// <summary>
        /// Gets or sets the wrappet expression list.
        /// </summary>
        /// <value>The wrapped expression list.</value>
        public List<CSPExprWrapper> WrappedExprList
        {
            get { return wrappedExprList; }
            set { wrappedExprList = value; }
        }

        /// <summary>
        /// Adds the wrappet expression.
        /// </summary>
        /// <param name="wrappedExpr">The wrappet expression.</param>
        /// <returns>The index the wrapped expression is placed on.</returns>
        [MethodImpl(MethodImplOptions.Synchronized)]
        public int AddWrappedExpr(CSPExprWrapper wrappedExpr)
        {
            wrappedExprList.Add(wrappedExpr);
            return wrappedExprList.Count - 1;
        }
    }
}