/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using Iesi.Collections;

namespace Casper.Data
{
    /// <summary>
    /// A class with variable and domain information, used by
    /// the <see cref="T:Csp"></see> class. 
    /// </summary>
    public class CasperVarDom
    {
        private int varID;
        private List<int> domainValues;
        private Set validValues;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CasperVarDom"/> class.
        /// </summary>
        /// <param name="varID">The variable ID.</param>
        /// <param name="domainValues">The domain values.</param>
        public CasperVarDom(int varID, List<int> domainValues)
        {
            this.varID = varID;
            this.domainValues = domainValues;
            this.validValues = new HybridSet();
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="T:CasperVarDom"/> class.
        /// </summary>
        /// <param name="varID">The variable ID.</param>
        public CasperVarDom(int varID)
        {
            this.varID = varID;
            this.validValues = new HybridSet();
        }

        /// <summary>
        /// Gets or sets the variable ID.
        /// </summary>
        /// <value>The variable ID.</value>
        public int VarID
        {
            get { return varID; }
            set { varID = value; }
        }

        /// <summary>
        /// Gets or sets the domain values.
        /// </summary>
        /// <value>The domain values.</value>
        public List<int> DomainValues
        {
            get { return domainValues; }
            set { domainValues = value; }
        }

        /// <summary>
        /// Gets or sets the valid values.
        /// </summary>
        /// <value>The valid values.</value>
        public Set ValidValues
        {
            get { return validValues; }
            set { validValues = value; }
        }

        /// <summary>
        /// Returns a "System.String" that represents the current "Casper.Data.CasperVarDom".
        /// </summary>
        /// <returns>
        /// A "System.String" that represents the current "Casper.Data.CasperVarDom".
        /// </returns>
        public override string ToString()
        {
            String r = varID + "\nDomain values:\n ";
            for (int i = 0; i < domainValues.Count; i++)
            {
                r += domainValues[i];
            }
            return r;
        }
    }
}