/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjrn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using Iesi.Collections;

using Casper.Exceptions;
using Casper.Algorithm.VariableOrdering;

namespace Casper.Data
{
    /// <summary>
    /// A class which represents the constraint graph of a csp problem.
    /// </summary>
    public class ConstraintGraph
    {
        //A table of adjacency list, where the key is each variable id and the values are its adjacency list
        Hashtable cGraph;
        
        /// <summary>
        /// Initializes a new instance of the <see cref="T:ConstraintGraph"/> class.
        /// </summary>
        public ConstraintGraph() 
        {
            this.cGraph = new Hashtable();
        }

        /// <summary>
        /// Gets the constraint graph.
        /// </summary>
        public Hashtable CGraph
        {
            get { return cGraph; }
        }

        /// <summary>
        /// Updates the adjacency list representation of the graph
        /// by adding all neighbours of a variable to its
        /// adjacency list. Creates the adjacency list if it doesn't exist.
        /// </summary>
        /// <param name="interconNodes">A list of interconnected nodes</param>
        /// <exception cref="T:Casper.Exceptions.CasperException"></exception>
        public void UpdateAdjacencyList(Set interconNodes)
        {
            IEnumerator ie = interconNodes.GetEnumerator();
            while(ie.MoveNext())
            {
                Set current = new HashedSet();
                current.Add(ie.Current);
                try
                {
                    AdjacencyList tempList = (AdjacencyList)cGraph[(int)ie.Current];
                    if (tempList == null)
                    {
                        tempList = new AdjacencyList();
                        cGraph.Add(ie.Current, tempList);//makes a place for the nodes adjacent to current node
                    }
                    tempList.AddAllVars(interconNodes.Minus(current));
                }
                catch (Exception e) 
                {
                    throw new CasperException("Error building adjacency list", e);
                }
            }
        }
    }
}
