using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;

using Casper.Data;

namespace Casper
{
    /// <summary>
    /// Interface for the consistency check
    /// </summary>
    public interface IConsistent
    {
        /// <summary>
        /// The constructor.
        /// </summary>
        /// <param name="assignedVars">Hashtable with the assigned variables.</param>
        /// <param name="currAss">Current assigned variable.</param>
        /// <param name="nextAss">Next assigned variable.</param>
        /// <returns></returns>
        Boolean Consistent(Hashtable assignedVars, IConsistentVar currAss, IConsistentVar nextAss);
        
        /// <summary>
        /// Executes the expressions using the assigned variable values.
        /// </summary>
        /// <param name="rule">The expression to execute.</param>
        /// <returns>Integer representation of expression result.</returns>
        int ConsistentCheckExpression(CSPExprVarInt rule);

        /// <summary>
        /// Executes the expressions using the assigned variable values.
        /// </summary>
        /// <param name="rule">The expression to execute.</param>
        /// <returns>Integer representation of expression result.</returns>
        int ConsistentCheckExpression(CSPExprVarBool rule);

        /// <summary>
        /// Executes the expressions using the assigned variable values.
        /// </summary>
        /// <param name="rule">The expression to execute.</param>
        /// <returns>Integer representation of expression result.</returns>
        int ConsistentCheckExpression(CSPExprVarEnum rule);

        /// <summary>
        /// Executes the expressions using the assigned variable values.
        /// </summary>
        /// <param name="rule">The expression to execute.</param>
        /// <returns>Integer representation of expression result.</returns>
        int ConsistentCheckExpression(CSPExprValue rule);

        /// <summary>
        /// Executes the expressions using the assigned variable values.
        /// </summary>
        /// <param name="rule">The expression to execute.</param>
        /// <returns>Integer representation of expression result.</returns>
        int ConsistentCheckExpression(CSPExprNotNeg rule);

        /// <summary>
        /// Executes the expressions using the assigned variable values.
        /// </summary>
        /// <param name="rule">The expression to execute.</param>
        /// <returns>Integer representation of expression result.</returns>
        int ConsistentCheckExpression(CSPExprBin rule);
    }
}
