﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;


namespace ClabGui
{
    /// <summary>
    /// A singleton for the different bitmaps representing
    /// the domain status for each domain value.
    /// </summary>
    class ParameterBitmapSingleton
    {
        private static ParameterBitmapSingleton instance;
        private List<Bitmap> bitmaps;
        public const int AVAILABLE = 0;
        public const int UNAVAIBALBE = 1;
        public const int VALID_USER = 2;
        public const int VALID_AUTO = 3;
        public const int INVALID = 4;

        /// <summary>
        /// Initializes a new instance of the <see cref="T:ParameterBitmapSingleton"/> class.
        /// </summary>
        protected ParameterBitmapSingleton()
        {
            bitmaps = new List<Bitmap>(5);
            bitmaps.Add(new Bitmap(GetType(), "Auxiliary.available.png"));
            bitmaps.Add(new Bitmap(GetType(), "Auxiliary.unavailable.png"));
            bitmaps.Add(new Bitmap(GetType(), "Auxiliary.valid_user.png"));
            bitmaps.Add(new Bitmap(GetType(), "Auxiliary.valid_auto.png"));
            bitmaps.Add(new Bitmap(GetType(), "Auxiliary.invalid.png"));
        }

        /// <summary>
        /// Creates a new instance if one doesn't exist.
        /// Return the existing instance otherwize.
        /// </summary>
        /// <returns></returns>
        public static ParameterBitmapSingleton Instance()
        {
            if (instance == null)
            {
                instance = new ParameterBitmapSingleton();
            }
            return instance;
        }

        /// <summary>
        /// Gets the bitmap representing the passed in int value.
        /// </summary>
        /// <param name="STATUS">The STATUS.</param>
        /// <returns></returns>
        public Bitmap GetBitmap(int STATUS)
        {
            if (STATUS > -1 && STATUS < 5)
            {
                return bitmaps[STATUS];
            }
            return null;
        }
    }
}
