﻿/*========================================================================
        Copyright (C) 2006 by Geir-Tore Lindsve, Torbjørn Meistad 
        and Yngve Raudberget, hereby refered to as "the authors".
                        All rights reserved

    Permission is hereby granted, without written agreement and without
    license or royalty fees, to use, reproduce, prepare derivative
    works, distribute, and display this software and its documentation
    for NONCOMMERCIAL RESEARCH AND EDUCATIONAL PURPOSES, provided 
    that (1) the above copyright notice and the following two paragraphs 
    appear in all copies of the source code and (2) redistributions, 
    including without limitation binaries, reproduce these notices in 
    the supporting documentation. 

    IN NO EVENT SHALL THE AUTHORS, OR DISTRIBUTORS OF THIS SOFTWARE 
    BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, 
    OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE 
    AND ITS DOCUMENTATION, EVEN IF THE AUTHORS OR ANY OF THE ABOVE 
    PARTIES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
    BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
    ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE NO
    OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
    MODIFICATIONS.
========================================================================*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Collections;

using Iesi.Collections;

namespace ClabGui
{
    class VarParameters
    {
        
        private ISet invalidVals;
        private ISet validVals;
        private ISet unavailableVals;
        private String userSelectedVal = "";
        private String autoSelectedVal = "";
        private Hashtable domainValStatus;
        private ParameterBitmapSingleton pms;

        public VarParameters(List<String> domainValues)
        {
            pms = ParameterBitmapSingleton.Instance();
            invalidVals = new HybridSet();
            validVals = new HybridSet();
            unavailableVals = new HybridSet();
            domainValStatus = new Hashtable();
            for (int i = 0; i < domainValues.Count; i++)
            {
                domainValStatus[domainValues[i]] = ParameterBitmapSingleton.AVAILABLE;
                validVals.Add(domainValues[i]);
            }
        
        }

        public void ValidDomainVals(List<String> validDomainVals)
        {
            ISet inputValidDomVals = new HybridSet();
            for (int i = 0; i < validDomainVals.Count; i++)
            {
                inputValidDomVals.Add(validDomainVals[i]);
            }

            ISet invalid = validVals.Minus(inputValidDomVals);
            invalidVals = invalidVals.Union(invalid);
            validVals = inputValidDomVals;

            if (validVals.Count == 1 && userSelectedVal.Equals(""))
            {
                autoSelectedVal = validDomainVals[0];
            }
            UpdateDomainValStatus();
        }

        private void UpdateDomainValStatus()
        {
            foreach (String val in unavailableVals)
            {
                domainValStatus[val] = ParameterBitmapSingleton.UNAVAIBALBE;
            }
            foreach (String val in invalidVals)
            {
                domainValStatus[val] = ParameterBitmapSingleton.INVALID;
            }
            if (!userSelectedVal.Equals(""))
            {
                domainValStatus[userSelectedVal] = ParameterBitmapSingleton.VALID_USER;
            }
            if (!autoSelectedVal.Equals(""))
            {
                domainValStatus[autoSelectedVal] = ParameterBitmapSingleton.VALID_AUTO;
            }
        }

        public void UserSelectedVal(String domainVal)
        {
            domainValStatus[domainVal] = ParameterBitmapSingleton.VALID_USER;
            userSelectedVal = domainVal;
            validVals.Remove(domainVal);
            unavailableVals = validVals;
            validVals = new HybridSet();
            validVals.Add(domainVal);
        }

        public Bitmap GetDomainValBitmap(String domainVal)
        {
            return pms.GetBitmap((int)domainValStatus[domainVal]);
        }

        public int GetStatus(String domainVal)
        {
            return (int) domainValStatus[domainVal];
        }

        
    }
}
